import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../model/response_bean.dart';
import '../../model/simple_device.dart';
import '../../provider/shared_data_provider.dart';
import '../../utils/net_util.dart';
import '../../utils/sp_util.dart';
import '../log/log_utils.dart';

const noDevicesTips = '未检测到连接设备';

/// 设备管理client
class DeviceManager extends StatefulWidget {
  @override
  _DeviceManagerState createState() => _DeviceManagerState();

  /// 获取当前连接的设备
  static Future<SimpleDevice> currentDevice() async {
    // 需要兼容本地缓存版本的current_device新老版本格式
    var jsonString = await SpUtil.getString("current_device");
    try {
      if (jsonString != null && jsonString.isNotEmpty) {
        Map map = jsonDecode(jsonString);
        return SimpleDevice.fromJson(map);
      }
    } catch (e) {
      print(e);
    }
    return SimpleDevice.empty;
  }
}

class _DeviceManagerState extends State<DeviceManager> {
  List<SimpleDevice> deviceList = [];
  bool isLoading = false;
  bool isFilterWebDevices = false;

  @override
  void initState() {
    super.initState();
    showDevices();
    checkWorkspace().then((result) {
      var skipUpdate = result[0];
      var path = result[1];
      var current = result[2];
      if (skipUpdate) {
        return Future.value([false, '', current]);
      }
      return SpUtil.setString(tPathSpKey, path).then((_) => [_, path, current]);
    }).then((result) {
      var success = result[0];
      var path = result[1];
      var previous = result[2];
      if (!success) {
        return;
      }
      var bar = SnackBar(
        content: Text('检测到新的工作目录，已更新：$path'),
        action: SnackBarAction(
          label: "点击撤销",
          textColor: Colors.white,
          onPressed: () {
            SpUtil.setString(tPathSpKey, previous).then(
                (onValue) => debugPrint('path has been reset to $previous'));
          },
        ),
      );
      Scaffold.of(context).showSnackBar(bar);
    });
  }

  Future<List> checkWorkspace() async {
    var response = await NetUtils.get('/api/path', withCode: true);
    var path = '';
    if (response is ResponseBean) {
      if (response.code == 1) {
        path = response.code == 0 ? (response.data as String) : '';
      }
    } else if (response is Response) {
      path = response.toString();
    }

    if (path == null || path == '') {
      // there is no updated path, ignore all
      return [true, '', ''];
    }
    var current = await SpUtil.getString(tPathSpKey);
    return [path == current, path, current];
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: Column(
        children: <Widget>[
          SizedBox(
            height: 20,
          ),
          Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              _refreshBtn(showDevices),
              _filterCheckBox(),
            ],
          ),
          SizedBox(
            height: 20,
          ),
          deviceList.isNotEmpty
              ? Column(
                  children: _generateDeviceList(deviceList),
                )
              : Padding(
                  padding: const EdgeInsets.all(15),
                  child: Text(
                    !isLoading ? noDevicesTips : "",
                    style: TextStyle(color: Colors.red, fontSize: 20),
                  ),
                ),
        ],
      ),
    );
  }

  List<Widget> _generateDeviceList(List<SimpleDevice> devices) {
    if (isFilterWebDevices) {
      devices = devices.where((d) {
        return "web" != d.platform;
      }).toList();
    }
    return List<Widget>.generate(devices.length, (i) {
      var currentDevice = provider().currentDevice();
      var id = currentDevice != null ? currentDevice.deviceId : "";
      var isSelectedOne = devices[i].deviceId == id;
      return DeviceItem(
        device: devices[i],
        isSelectedOne: isSelectedOne,
        click: (d) {
          setState(() {
            currentDevice = d;
            provider().changeDevice(currentDevice);
            String jsonString = jsonEncode(d.toJson());
            SpUtil.setString("current_device", jsonString);
          });
        },
      );
    }).toList();
  }

  SharedDataProvider provider() => Provider.of<SharedDataProvider>(context, listen: false);

  /// 刷新按钮
  Widget _refreshBtn(Function onClick) {
    return GestureDetector(
      onTap: onClick,
      child: Container(
        color: Colors.green,
        height: 50,
        width: 150,
        margin: EdgeInsets.symmetric(horizontal: 15),
        padding: EdgeInsets.symmetric(horizontal: 15),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Text(
              isLoading ? "加载中..." : "刷新设备",
              style: TextStyle(color: Colors.white),
            ),
            isLoading
                ? SizedBox(
                    width: 15,
                    height: 15,
                    child: CircularProgressIndicator(),
                  )
                : Text('')
          ],
        ),
      ),
    );
  }

  Widget _filterCheckBox() {
    return Container(
      height: 50,
      width: 150,
      margin: EdgeInsets.symmetric(horizontal: 15),
      color: Colors.lightGreen,
      child: Row(
        children: <Widget>[
          Checkbox(
              value: isFilterWebDevices,
              onChanged: (b) {
                setState(() {
                  isFilterWebDevices = b;
                });
              }),
          Text(
            "不看web设备",
            style: TextStyle(color: Colors.white),
          )
        ],
      ),
    );
  }

  void showDevices() {
    setState(() {
      isLoading = true;
    });
    fetchDeviceList().then((map) {
      deviceList = map['devices'];
      var currentDevice = map['selected'];
      provider().changeDevice(currentDevice);
      if (!mounted) {
        return;
      }
      setState(() {
        isLoading = false;
      });
    }).catchError(() {
      debugPrint('device parse error');
      setState(() {
        isLoading = false;
      });
    });
  }

  /// 拉取设备列表
  Future<Map> fetchDeviceList() async {
    var response = await NetUtils.get("/api/device/list");
    debugPrint("decvice => $response");
    var pre = await DeviceManager.currentDevice();
    debugPrint("pre decvice => $pre");
    var devices = SimpleDevice.parseDevices(response);
    debugPrint("decvices => $devices");
    var selected = null;
    if (devices.isNotEmpty) {
      selected = devices.firstWhere((item) => item.deviceId == pre.deviceId,
          orElse: () => devices[0]);
      String jsonString = jsonEncode(selected.toJson());
      SpUtil.setString("current_device", jsonString);
      logger.info("device list count : ${devices.length}");
    } else {
      SpUtil.setString("current_device", "");
      logger.warning("device list empty");
    }
    return {"selected": selected, 'devices': devices};
  }
}

/// 设备item
class DeviceItem extends StatelessWidget {
  final SimpleDevice device;
  final bool isSelectedOne;
  final DeviceItemClick click;

  const DeviceItem({Key key, this.device, this.click, this.isSelectedOne})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var color = isSelectedOne ? Colors.green : Colors.grey;
    var borderWidth = isSelectedOne ? 3.0 : 1.0;
    var logo = Icons.device_unknown;
    switch (device.platform) {
      case "android":
        logo = Icons.android;
        break;
      case "ios":
        logo = Icons.phone_iphone;
        break;
      case "web":
        logo = Icons.web;
        break;
    }

    return GestureDetector(
      onTap: () {
        click(device);
      },
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          ConstrainedBox(
            constraints:
                BoxConstraints(minWidth: 200, maxWidth: double.infinity),
            child: Container(
              alignment: Alignment.center,
              margin: EdgeInsets.symmetric(vertical: 10),
              decoration: BoxDecoration(
                  border: Border.all(color: color, width: borderWidth)),
              padding: EdgeInsets.symmetric(vertical: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Icon(
                    logo,
                    color: Colors.white,
                    size: 20,
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 10),
                    child: Text(
                      device.deviceName,
                      style: TextStyle(color: Colors.white, fontSize: 20),
                    ),
                  ),
                ],
              ),
            ),
          ),
          SizedBox(
            width: 50,
            height: 50,
            child: isSelectedOne
                ? Icon(
                    Icons.done,
                    color: Colors.green,
                  )
                : Text(''),
          )
        ],
      ),
    );
  }
}

typedef DeviceItemClick = void Function(SimpleDevice device);
